﻿define(['services/logger'], function (logger) {
    var vm = {
        title: "jQWidgets Example",
        items: ko.observableArray([]),
        manager: null,
        activate: function () {
            logger.log(this.title + ' View Activated', null, this.title, true);
            if (this.items().length > 0)
            {
                return;
            }
            // Read functionality
            this.manager = new breeze.EntityManager("/api/todos");
            var query = breeze.EntityQuery
               .from("Todos")
               .orderBy("CreatedAt");
            var me = this;
            return this.manager.executeQuery(query).then(function (data) {
                data.results.forEach(function (currentItem) {
                    me.items().push({ Description: currentItem.Description(), IsDone: currentItem.IsDone() });
                });
            }).fail(function (e) {
                logger.log("Error Loading Items", null, this.title, true);
            });
        },
        attached: function (view) {
            $("#jqxDataTable").jqxDataTable(
                {
                    theme: "bootstrap",
                    width: "400px",
                    altrows: true,
                    sortable: true,
                    editable: true,
                    columns: [
                      { text: "Description", dataField: "Description", width: 300, editable: false, columngroup: "toDoList" },
                      { text: 'Done', dataField: 'IsDone', width: 100, columngroup: "toDoList" }
                    ],
                    columnGroups:
                    [
                      { text: 'To Do List', align: 'center', name: 'toDoList' }
                    ]
                });
            var me = this;
            // Update functionality
            $('#jqxDataTable').on('rowEndEdit', function (event) {
                var args = event.args;
                var row = args.row;
                var rowIndex = args.index;

                var IsDone = row.IsDone === "true" ? true : false;

                var entities = me.manager.getEntities();
                entities[rowIndex].IsDone(IsDone);
                me.saveChanges();
            });
            return true;
        },
        // Create functionality
        addNewItem: function () {
            var value = $("#newItem").val();
            var entities = this.manager.getEntities();
            var entityId = entities[entities.length - 1].Id() + 1;
            var rows = $("#jqxDataTable").jqxDataTable('getRows');
            var rowId = rows[rows.length - 1].uid + 1;
            $("#jqxDataTable").jqxDataTable('addRow', rowId, { Description: value, IsDone: false });
            this.manager.createEntity('TodoItem', { Id: entityId, Description: value, CreatedAt: new Date(), IsDone: false });
            this.saveChanges();
            $("#newItem").val("");
        },
        // Delete functionality
        deleteItems: function () {
            var selectedRows = $("#jqxDataTable").jqxDataTable('getSelection');
            var entities = this.manager.getEntities();
            for (var i = 0; i < selectedRows.length; i++)
            {
                rowId = selectedRows[i].uid;
                $("#jqxDataTable").jqxDataTable('deleteRow', rowId);
                entities[rowId].entityAspect.setDeleted();
            }
            this.saveChanges();
        },
        saveChanges: function () {
            if (this.manager.hasChanges())
            {
                this.manager.saveChanges().then(function () {
                    logger.log("Changes Saved", null, this.title, true);
                }).fail(function () {
                    logger.log("Error Saving Changes", null, this.title, true);
                });
            }
        }
    };

    return vm;
});